unit PtSrc;

interface

(* Embracer Point Source Module
   Copyright (c) 1997 by William E. Sorensen.
   May be distributed for non-commercial purposes.
*)

(* Notes on the TPointSource class:
   1) FPSObject is just a pointer.
      The actual object referenced is not stored, created, or freed here.
      Be sure that it remains valid (in scope, etc.).
      Ditto for FDestDotEdit.
   2) Assigning a value to DestDotEdit also changes the value of FPSObject.
   3) Methods and properties may raise EPointSourceError exceptions.
      Be sure to use try.finally blocks.
*)

uses
  SysUtils, StdCtrls, ExtCtrls, DotEdit;

type
  TPointSource = class
  private
    FPSObject    : TObject;
    FDestDotEdit : TDotEdit;
    procedure SetDestDotEdit(DE: TDotEdit);
    procedure FindPSObject;
    function GetPSValue: Byte;
    procedure SetPSValue(Value: Byte);
    function CalcPointCost : Byte;
  public
    constructor Create;
    procedure Assign(Source: TPointSource);
    procedure ApplyPointToDot;
    procedure ReturnPointToSource;
    property DestDotEdit : TDotEdit
      read FDestDotEdit
      write SetDestDotEdit;
    property PSValue : Byte
      read GetPSValue
      write SetPSValue;
  end;

  EPointSourceError = class(Exception);

implementation

uses
  Main;

procedure TPointSource.SetDestDotEdit(DE: TDotEdit);
begin
  FDestDotEdit := DE;
  FindPSObject;
end;

procedure TPointSource.FindPSObject;
begin
  FPSObject := nil;

  if Assigned(FDestDotEdit) then
    with FDestDotEdit do
      begin
        if MAIN.MainForm.FreebiePointsPanel.Enabled then
          FPSObject := MAIN.MainForm.FreebiePointsPanel
        else if Parent is TGroupBox then
          with TGroupBox(Parent) do
            if Name = 'GBPhysical' then
              FPSObject := MAIN.MainForm.PhysicalPointsPanel
            else if Name = 'GBSocial' then
              FPSObject := MAIN.MainForm.SocialPointsPanel
            else if Name = 'GBMental' then
              FPSObject := MAIN.MainForm.MentalPointsPanel
            else if Name = 'GBTalents' then
              FPSObject := MAIN.MainForm.TalentsPointsPanel
            else if Name = 'GBSkills' then
              FPSObject := MAIN.MainForm.SkillsPointsPanel
            else if Name = 'GBKnowledge' then
              FPSObject := MAIN.MainForm.KnowledgePointsPanel
            else if Name = 'GBDisciplines' then
              FPSObject := MAIN.MainForm.DisciplinesPointsPanel
            else if Name = 'GBBackgrounds' then
              FPSObject := MAIN.MainForm.BackgroundsPointsPanel
            else if Name = 'GBVirtues' then
              FPSObject := MAIN.MainForm.VirtuesPointsPanel;
      end;

  if not Assigned(FPSObject) then
    raise EPointSourceError.Create('No Point Source found');
end;

function TPointSource.GetPSValue: Byte;
begin
  Result := 0;

  if not Assigned(FPSObject) then
    raise EPointSourceError.Create('No Point Source was found');

  if FPSObject is TPanel then
    Result := StrToIntDef(TPanel(FPSObject).Caption,0);
end;

procedure TPointSource.SetPSValue(Value: Byte);
begin
  if not Assigned(FPSObject) then
    raise EPointSourceError.Create('No Point Source was found');

  if FPSObject is TPanel then
    TPanel(FPSObject).Caption := IntToStr(Value);
end;

function TPointSource.CalcPointCost : Byte;
begin
  Result := 0;

  if not Assigned(FDestDotEdit) then
    raise EPointSourceError.Create('No Destination has been set');

  if MAIN.MainForm.FreebiePointsPanel.Enabled then
    begin
      with FDestDotEdit do
        if Name = 'DEWillpower' then
          Result := 2
        else if Name = 'DEHumanity' then
          Result := 1
        else if Parent is TGroupBox then
          with TGroupBox(Parent) do
            if Name = 'GBDisciplines' then
              Result := 7
            else if (Name = 'GBPhysical') or
                    (Name = 'GBSocial') or
                    (Name = 'GBMental') then  { Attributes }
              Result := 5
            else if (Name = 'GBTalents') or
                    (Name = 'GBSkills') or
                    (Name = 'GBKnowledge') then  { Abilities }
              Result := 2
            else if Name = 'GBVirtues' then
              Result := 2
            else if Name = 'GBBackgrounds' then
              Result := 1;
    end
  else
    Result := 1;

  if Result = 0 then
    raise EPointSourceError.Create('Unable to calculate point cost');
end;

constructor TPointSource.Create;
begin
  FPSObject    := nil;
  FDestDotEdit := nil;
end;

procedure TPointSource.Assign(Source: TPointSource);
begin
  if Assigned(Source) then
    begin
      FPSObject    := Source.FPSObject;
      FDestDotEdit := Source.FDestDotEdit;
    end
  else
    begin
      FPSObject    := nil;
      FDestDotEdit := nil;
    end;
end;

procedure TPointSource.ApplyPointToDot;
var
  PointCost : Byte;
begin
  if not Assigned(FDestDotEdit) then
    raise EPointSourceError.Create('No Destination has been set');

  if not Assigned(FPSObject) then
    raise EPointSourceError.Create('No Point Source was found');

  PointCost := CalcPointCost;

  if PSValue < PointCost then
    raise EPointSourceError.Create('Not enough points are available');

  with FDestDotEdit do
    begin
      if Dots = MaxDots then
        raise EPointSourceError.Create('Trait is already at maximum');

      (* Initial Ability scores are restricted to INIT_ABIL_MAX.
         Note that with Freebie Points, FPSObject will be different,
         so this restriction will be lifted (as it should be).
       *)
      if (not MAIN.MainForm.FreebiePointsPanel.Enabled) and
         (Dots = INIT_ABIL_MAX) and (FPSObject is TPanel) then
        with TPanel(FPSObject) do
          if (Name = 'TalentsPointsPanel') or
             (Name = 'SkillsPointsPanel') or
             (Name = 'KnowledgePointsPanel') then
            raise EPointSourceError.Create('Only Freebie Points can raise ' +
                                           'an Ability above ' +
                                           IntToStr(INIT_ABIL_MAX));

      PSValue := PSValue - PointCost;
      Dots    := Dots + 1;
    end;
end;

procedure TPointSource.ReturnPointToSource;
begin
 if not Assigned(FDestDotEdit) then
    raise EPointSourceError.Create('No Destination has been set');

  if not Assigned(FPSObject) then
    raise EPointSourceError.Create('No Point Source was found');

  with FDestDotEdit do
    begin
      if Dots < 1 then
        raise EPointSourceError.Create('No points left');

      Dots    := Dots - 1;
      PSValue := PSValue + CalcPointCost;
    end;
end;

end.
