unit undo;

interface

(* Embracer Undo Module
   Copyright (c) 1997 by William E. Sorensen.
   May be distributed for non-commercial purposes.
*)

(* Notes on the TUndoStack class:
   1) Push makes a copy of the original PointSource.
      Therefore, the original PointSource object can be freed at any time
      (as long as the objects it references remain valid).
   2) Methods may raise EUndoStackError exceptions.
*)

uses
  SysUtils, Classes, StdCtrls, ComCtrls, PtSrc, DotEdit;

type
  TUndoStack = class
  private
    FUndoList : TList;
    procedure SwitchToRightPage(DE: TDotEdit);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    procedure Push(PointSource: TPointSource);
    procedure PopAndUndo;

  end;

  EUndoStackError = class(Exception);

implementation

uses
  Main;

procedure TUndoStack.SwitchToRightPage(DE: TDotEdit);
{ only Freebie Points will cause a switch }
var
  Dummy : Boolean;
begin
  if Assigned(DE) and
     (DE.Parent is TGroupBox) and
     (DE.Parent.Parent is TTabSheet) then
    with TTabSheet(DE.Parent.Parent) do
      if not Showing then
        begin
          { The Showing check doesn't work on the DotEdit. }
          Dummy := True;  { var parameter }
          { Due to the Showing check, we know the page is actually changing. }
          MAIN.MainForm.PCMainChanging(Self,Dummy);
          Show;
          MAIN.MainForm.PCMainChange(Self);
        end;
end;

constructor TUndoStack.Create;
begin
  FUndoList := TList.Create;
end;

destructor TUndoStack.Destroy;
begin
  try
    Clear;  { frees objects in list }
  finally
    FUndoList.Free;
    inherited Destroy;
  end;
end;

procedure TUndoStack.Clear;
var
  i: Integer;
begin
  with FUndoList do
    begin
      for i := (Count - 1) downto 0 do
        TPointSource(Items[i]).Free;

      Clear;
    end;
end;

procedure TUndoStack.Push(PointSource: TPointSource);
var
  PSCopy : TPointSource;
begin
  PSCopy := TPointSource.Create;
  try
    PSCopy.Assign(PointSource);
    FUndoList.Add(PSCopy);
  except
    { try..except, not try..finally - normally, PSCopy will be freed later }
    PSCopy.Free;
    raise;
  end;
end;

procedure TUndoStack.PopAndUndo;
var
  PoppedPS: TPointSource;
begin
  { checking Count, as Last would generate exception if no items in list }
  if FUndoList.Count = 0 then
    raise EUndoStackError.Create('Nothing to undo');

  PoppedPS := TPointSource(FUndoList.Last);

  with PoppedPS do
    try
      { must switch page first if dealing with Final/Freebies }
      SwitchToRightPage(DestDotEdit);
      
      ReturnPointToSource;

      { These use the read directive, not the write, of property DestDotEdit! }
      DestDotEdit.SetFocus;
      DestDotEdit.SelectAll;
    finally
      FUndoList.Remove(PoppedPS);
      Free;
    end;
end;

end.
